/// @docImport 'package:flutter/services.dart';
/// @docImport 'package:consoledoctor_barcode_scanner/src/barcode_scanner_controller.dart';
library;

import 'package:consoledoctor_barcode_scanner/src/enums/barcode_scanner_error_code.dart';

/// This class represents an exception thrown by the [BarcodeScannerController].
class BarcodeScannerException implements Exception {
  /// Construct a new [BarcodeScannerException] instance.
  const BarcodeScannerException({
    required this.errorCode,
    this.errorDetails,
  });

  /// The error code of the exception.
  final BarcodeScannerErrorCode errorCode;

  /// The additional error details that came with the [errorCode].
  final BarcodeScannerErrorDetails? errorDetails;

  @override
  String toString() {
    if (errorDetails != null && errorDetails?.message != null) {
      return 'BarcodeScannerException(${errorCode.name}, ${errorDetails?.message})';
    }
    return 'BarcodeScannerException(${errorCode.name})';
  }
}

/// The raw error details for a [BarcodeScannerException].
class BarcodeScannerErrorDetails {
  /// Construct a new [BarcodeScannerErrorDetails] instance.
  const BarcodeScannerErrorDetails({
    this.code,
    this.details,
    this.message,
  });

  /// The error code from the [PlatformException].
  final String? code;

  /// The details from the [PlatformException].
  final Object? details;

  /// The error message from the [PlatformException].
  final String? message;
}

/// This class represents an exception thrown by the [BarcodeScannerController]
/// when a barcode scanning error occurs when processing an input frame.
class BarcodeScannerBarcodeException implements Exception {
  /// Creates a new [BarcodeScannerBarcodeException] with the given error message.
  const BarcodeScannerBarcodeException(this.message);

  /// The error message of the exception.
  final String? message;

  @override
  String toString() {
    if (message?.isNotEmpty ?? false) {
      return 'BarcodeScannerBarcodeException($message)';
    }

    return 'BarcodeScannerBarcodeException(Could not detect a barcode in the input image.)';
  }
}
