import 'package:flutter/services.dart';
import 'package:consoledoctor_barcode_scanner/src/barcode_scanner_controller.dart';

/// This enum defines the different error codes for the mobile scanner.
enum BarcodeScannerErrorCode {
  /// The controller was already started.
  ///
  /// The controller should be stopped using [BarcodeScannerController.stop],
  /// before restarting it.
  controllerAlreadyInitialized,

  /// The controller was used after being disposed.
  controllerDisposed,

  /// The controller was used
  /// while it was not yet initialized using [BarcodeScannerController.start].
  controllerUninitialized,

  /// A generic error occurred.
  ///
  /// This error code is used for all errors that do not have a specific error code.
  genericError,

  /// The permission to use the camera was denied.
  permissionDenied,

  /// Scanning is unsupported on the current device.
  unsupported;

  /// Convert the given [PlatformException.code] to a [BarcodeScannerErrorCode].
  factory BarcodeScannerErrorCode.fromPlatformException(
    PlatformException exception,
  ) {
    // The following error code mapping should be kept in sync with their native counterparts.
    // These are located in `MobileScannerErrorCodes.kt` and `MobileScannerErrorCodes.swift`.
    return switch (exception.code) {
      // In case the scanner was already started, report the right error code.
      // If the scanner is already starting,
      // this error code is a signal to the controller to just ignore the attempt.
      'BARCODE_SCANNER_ALREADY_STARTED_ERROR' =>
        BarcodeScannerErrorCode.controllerAlreadyInitialized,
      // In case no cameras are available, using the scanner is not supported.
      'BARCODE_SCANNER_NO_CAMERA_ERROR' => BarcodeScannerErrorCode.unsupported,
      'BARCODE_SCANNER_CAMERA_PERMISSION_DENIED' =>
        BarcodeScannerErrorCode.permissionDenied,
      _ => BarcodeScannerErrorCode.genericError,
    };
  }
}
